#include <DirectX\6.0\inc\dsound.h>

#include "data.h"

typedef struct
{
	BYTE        RIFF[4];          // "RIFF"
	DWORD       dwSize;           // Size of data to follow
	BYTE        WAVE[4];          // "WAVE"
	BYTE        fmt_[4];          // "fmt "
	DWORD       dw16;             // 16
	WORD        wOne_0;           // 1
	WORD        wChnls;           // Number of Channels
	DWORD       dwSRate;          // Sample Rate
	DWORD       BytesPerSec;      // Sample Rate
	WORD        wBlkAlign;        // 1
	WORD        BitsPerSample;    // Sample size
	BYTE        DATA[4];          // "DATA"
	DWORD       dwDSize;          // Number of Samples
} Waveheader;

typedef struct {
	char           name[13];
   long           size;
   unsigned char  compressiontype;
} Rcs_Info;

#define MAX_SOUNDS 4

#define DSVOLUME_TO_DB(volume) ((DWORD)(-30*(100 - volume)))

LPDIRECTSOUND lpDS;
LPDIRECTSOUNDBUFFER lpDSBPrimary;
LPDIRECTSOUNDBUFFER lpDSBSounds[MAX_SOUNDS];

bool DS_Init(void);
bool DS_Fini(void);
bool DS_LoadWAV(char *filename, int num);
bool DS_LoadWAVRcs(LPCTSTR rc, int num);
bool DS_Play(int num);
bool DS_Stop(int num);
bool DS_SetVolume(int num,int vol);
bool DS_SetFreq(int num,int freq);
bool DS_SetPan(int num,int pan);
int  DS_Status(int num);
void DebugMessage(char *msg);

bool DS_Init(void)
{

DSBUFFERDESC dsbdesc;

   // create a directsound object
   if(DirectSoundCreate(NULL, &lpDS, NULL)!=DS_OK ) return false;
   // set cooperation level
   if(lpDS->SetCooperativeLevel((HWND)main_window_handle,DSSCL_NORMAL)!=DS_OK) return false;
   // Create a Primary Sound Buffer.
	dsbdesc.dwSize              = sizeof(DSBUFFERDESC);
	dsbdesc.dwFlags             = DSBCAPS_PRIMARYBUFFER;
	dsbdesc.dwBufferBytes       = 0;
	dsbdesc.lpwfxFormat         = NULL;
   lpDS->CreateSoundBuffer(&dsbdesc, &lpDSBPrimary, NULL);
   // return sucess
   return true;
}


bool DS_Fini(void)
{

int index;

   // now release all sound buffers
   for(index=0; index<MAX_SOUNDS; index++)
	   if(lpDSBSounds[index]) {
		   DS_Stop(index);
		   lpDSBSounds[index]->Release();
         }
   // now release the Primary Surface.
	if (lpDSBPrimary) {
      lpDSBPrimary->Release();
      lpDSBPrimary = NULL;
      }
   // now release the directsound interface itself
   if (lpDS) {
      lpDS->Release();
      lpDS = NULL;
      }
   // return success
   return true;
}

bool DS_LoadWAV(char *filename, int num)
{

PCMWAVEFORMAT pcmwf;
DSBUFFERDESC  dsbdesc;
Waveheader    Wavhdr;
LPVOID        pData1;
DWORD         dwData1Size;
LPVOID        pData2;
DWORD         dwData2Size;
HRESULT       rval;
FILE*         fp;

	// Open the wave file
	fp = fopen(filename, "rb");
	if (fp == NULL) return false;

	// Read in the wave header
	if (fread(&Wavhdr, sizeof(Wavhdr), 1, fp) != 1) {
		fclose(fp);
		return false;
	   }

	// Set up wave format structure.
	memset(&pcmwf, 0, sizeof(PCMWAVEFORMAT));
	pcmwf.wf.wFormatTag         = WAVE_FORMAT_PCM;
	pcmwf.wf.nChannels          = Wavhdr.wChnls;
	pcmwf.wf.nSamplesPerSec     = Wavhdr.dwSRate;
	pcmwf.wf.nBlockAlign        = (WORD)Wavhdr.wBlkAlign;
	pcmwf.wf.nAvgBytesPerSec    = pcmwf.wf.nSamplesPerSec * pcmwf.wf.nBlockAlign;
	pcmwf.wBitsPerSample        = (WORD)Wavhdr.BitsPerSample;

	// Set up DSBUFFERDESC structure.
	memset(&dsbdesc, 0, sizeof(DSBUFFERDESC));
	dsbdesc.dwSize              = sizeof(DSBUFFERDESC);
	dsbdesc.dwFlags             = DSBCAPS_STATIC | DSBCAPS_CTRLDEFAULT | DSBCAPS_GETCURRENTPOSITION2;
	dsbdesc.dwBufferBytes       = Wavhdr.dwDSize;
	dsbdesc.lpwfxFormat         = (LPWAVEFORMATEX)&pcmwf;

	lpDS->CreateSoundBuffer(&dsbdesc, &lpDSBSounds[num], NULL);

	// Read the data for the wave file into the sound buffer

	// Lock data in buffer for writing
	rval = lpDSBSounds[num]->Lock(0, Wavhdr.dwDSize, &pData1, &dwData1Size, &pData2, &dwData2Size, DSBLOCK_FROMWRITECURSOR);
	if (rval != DS_OK) return false;

	// Read in first chunk of data
	if (dwData1Size > 0) if (fread(pData1, dwData1Size, 1, fp) != 1) return false;

	// Read in second chunk if necessary
	if (dwData2Size > 0) if (fread(pData2, dwData2Size, 1, fp) != 1) return false;

	// Unlock data in buffer
	rval = lpDSBSounds[num]->Unlock(pData1, dwData1Size, pData2, dwData2Size);
	if (rval != DS_OK) return false;

	// Close out the wave file
	fclose(fp);

	return true;
}

bool DS_LoadWAVRcs(LPCTSTR rc, int num)
{

PCMWAVEFORMAT pcmwf;
Waveheader    Wavhdr;
DSBUFFERDESC  dsbdesc;
LPVOID        pData1;
DWORD         dwData1Size;
LPVOID        pData2;
DWORD         dwData2Size;
HRESULT       rval;
Rcs_Info      RcsInfo;
FILE*         fp;

	// Open the wave file
	fp = fopen("mraks.aud", "rb");
	if (fp == NULL) return false;

   if (fread(&RcsInfo, sizeof(Rcs_Info), 1, fp) != 1) {
   	fclose(fp);
   	return false;
   }

   while (strcmp(RcsInfo.name, rc)!=0) {
  		if (fseek(fp, RcsInfo.size, SEEK_CUR) != 0) {
         fclose(fp);
			return false;
	   	}
		if (fread(&RcsInfo, sizeof(Rcs_Info), 1, fp) != 1) {
			fclose(fp);
			return false;
	   	}
      }

	// Read in the wave header
	if (fread(&Wavhdr, sizeof(Wavhdr), 1, fp) != 1) {
		fclose(fp);
		return false;
	   }

	// Set up wave format structure.
	memset(&pcmwf, 0, sizeof(PCMWAVEFORMAT));
	pcmwf.wf.wFormatTag         = WAVE_FORMAT_PCM;
	pcmwf.wf.nChannels          = Wavhdr.wChnls;
	pcmwf.wf.nSamplesPerSec     = Wavhdr.dwSRate;
	pcmwf.wf.nBlockAlign        = (WORD)Wavhdr.wBlkAlign;
	pcmwf.wf.nAvgBytesPerSec    = pcmwf.wf.nSamplesPerSec * pcmwf.wf.nBlockAlign;
	pcmwf.wBitsPerSample        = (WORD)Wavhdr.BitsPerSample;

	// Set up DSBUFFERDESC structure.
	memset(&dsbdesc, 0, sizeof(DSBUFFERDESC));
	dsbdesc.dwSize              = sizeof(DSBUFFERDESC);
	dsbdesc.dwFlags             = DSBCAPS_STATIC | DSBCAPS_CTRLDEFAULT | DSBCAPS_GETCURRENTPOSITION2;
	dsbdesc.dwBufferBytes       = Wavhdr.dwDSize;
	dsbdesc.lpwfxFormat         = (LPWAVEFORMATEX)&pcmwf;

	lpDS->CreateSoundBuffer(&dsbdesc, &lpDSBSounds[num], NULL);

	// Read the data for the wave file into the sound buffer

	// Lock data in buffer for writing
	rval = lpDSBSounds[num]->Lock(0, Wavhdr.dwDSize, &pData1, &dwData1Size, &pData2, &dwData2Size, DSBLOCK_FROMWRITECURSOR);
	if (rval != DS_OK) return false;

	// Read in first chunk of data
	if (dwData1Size > 0) if (fread(pData1, dwData1Size, 1, fp) != 1) return false;

	// Read in second chunk if necessary
	if (dwData2Size > 0) if (fread(pData2, dwData2Size, 1, fp) != 1) return false;

	// Unlock data in buffer
	rval = lpDSBSounds[num]->Unlock(pData1, dwData1Size, pData2, dwData2Size);
	if (rval != DS_OK) return false;

	return true;
}

bool DS_Play(int num)
{
   if (lpDSBSounds[num]) {
   	// reset position to start
   	if(lpDSBSounds[num]->SetCurrentPosition(0)!=DS_OK) return false;
   	// play sound
   	if(lpDSBSounds[num]->Play(0,0,NULL)!=DS_OK) return false;
   	return true;
      }
   return false;
}

bool DS_Stop(int num)
{

	if (lpDSBSounds[num]) {
   	lpDSBSounds[num]->Stop();
   	lpDSBSounds[num]->SetCurrentPosition(0);
   	return true;
      }
   return false;
}


bool DS_SetVolume(int num,int vol)
{

   if (lpDSBSounds[num]) {
   	if(lpDSBSounds[num]->SetVolume(DSVOLUME_TO_DB(vol))!=DS_OK) return false;
   	return true;
      }
   return false;
}

bool DS_SetFreq(int num,int freq)
{

	if (lpDSBSounds[num]) {
   	if(lpDSBSounds[num]->SetFrequency(freq)!=DS_OK) return false;
   	return true;
      }
   return false;
}

bool DS_SetPan(int num,int pan)
{

	if (lpDSBSounds[num]) {
   	if (lpDSBSounds[num]->SetPan(pan)!=DS_OK) return false;
   	return true;
      }
   return false;
}

int DS_Status(int num)
{

ULONG status;

   if(lpDSBSounds[num]) {
	   // get the status
	   lpDSBSounds[num]->GetStatus(&status);
	   // return the status
	   return(status);
   	}
   else return -1;
}

void DebugMessage(char *msg)
{

   MessageBox( main_window_handle, msg, "Debug Message", MB_OK );
}

