/*********************************************************************************
*                                                                                *
*  MRAKS:        Mindlessly Running Around Killing Stuff.                        *
*                                                                                *
*  Version:      0.1 BETA 6                                                       *
*                                                                                *
*  Author:       Tim Wright                                                      *
*                                                                                *
*  Purpose:      To test my skills and to have a little fun.                     *
*                                                                                *
*  Useage:       Make sure DirectX 3.0 is installed then run MRAKS.exe           *
*                                                                                *
*  Limitations:  Generally sucks.                                                *
*                                                                                *
*  Future Updates:                                                               *
*                Better Pickups.                                                 *
*                                                                                *
*  Updates:      None.                                                           *
*                                                                                *
*********************************************************************************/

/////////***************************INCLUDES****************************//////////
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <windowsx.h>
#include <mmsystem.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <stdio.h>
#include <DirectX\6.0\inc\ddraw.h>
#include <DirectX\6.0\Utilities\ddutil.cpp>

#include "DS_Funct.c"
#include "config.cpp"
#include "Timer.cpp"
#include "CDMusic.c"

/*********************************************************************************
*  FiniObjects:  Release all Memory, Surfaces, Devices.                          *
*  Returns:      None.                                                           *
*********************************************************************************/
void FiniObjects( void )
{

	DD_Fini();
   DS_Fini();
   Close_Music();
   StopCD();
}

/*********************************************************************************
*  WindowProc:  Windows Message Handler.                                         *
*  Returns:     Windows Message.                                                 *
*********************************************************************************/
long FAR PASCAL WindowProc( HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam )
{

   switch( message ) {
      case WM_SETCURSOR:
         SetCursor(NULL);         // Disable the mouse.
         return TRUE;

      case WM_CREATE:
         break;

      case WM_DESTROY:
         FiniObjects();
         PostQuitMessage( 0 );
         break;

		default:break;
      }

   return DefWindowProc(hWnd, message, wParam, lParam);

}

/*********************************************************************************
*  DO_Init:   Create the Window, Direct Draw Object and Surfaces.                *
*  Returns:    TRUE on Sucsess.                                                  *
*********************************************************************************/
static bool DO_Init( HINSTANCE hInstance, int nCmdShow )
{

HWND                hwnd;
WNDCLASS            wc;
DDSURFACEDESC       ddsd;
DDSCAPS             ddscaps;
HRESULT             ddrval;

   // Set up and register window class
   wc.style = CS_HREDRAW | CS_VREDRAW;
   wc.lpfnWndProc = WindowProc;
   wc.cbClsExtra = 0;
   wc.cbWndExtra = 0;
   wc.hInstance = hInstance;
   wc.hIcon = LoadIcon( hInstance, MAKEINTRESOURCE(300));
   wc.hCursor = LoadCursor( NULL, IDC_ARROW );
   wc.hbrBackground = GetStockBrush(BLACK_BRUSH);
   wc.lpszMenuName = NAME;
   wc.lpszClassName = NAME;
   RegisterClass( &wc );

   // Create a window
   hwnd = CreateWindowEx(
      0,
      NAME,
      TITLE,
      WS_POPUP,
      0,
      0,
      GetSystemMetrics(SM_CXSCREEN),
      GetSystemMetrics(SM_CYSCREEN),
      NULL,
      NULL,
      hInstance,
      NULL );

   if( !hwnd ) return false;

   // Store the Main Window Handle.
	main_window_handle = hwnd;

   // Strut our stuff.
   ShowWindow( hwnd, nCmdShow );
   UpdateWindow( hwnd );

   // Load Configurations
   Load_Config();

   // Create the main DirectDraw object.
   ddrval = DirectDrawCreate( NULL, &lpDD, NULL );
   if( ddrval != DD_OK ) return DD_InitFail(hwnd);

   // Get exclusive mode.
   ddrval = lpDD->SetCooperativeLevel( hwnd, DDSCL_EXCLUSIVE | DDSCL_FULLSCREEN | DDSCL_ALLOWREBOOT );
   if( ddrval != DD_OK ) return DD_InitFail(hwnd);

   // Set the video mode to 640x480x?.
   ddrval = lpDD->SetDisplayMode( 640, 480, config.bpp);
   if( ddrval != DD_OK ) return DD_InitFail(hwnd);

   // Create the primary surface with 1 back buffer.
   ddsd.dwSize = sizeof( ddsd );
   ddsd.dwFlags = DDSD_CAPS | DDSD_BACKBUFFERCOUNT;
   ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE | DDSCAPS_FLIP | DDSCAPS_COMPLEX;
   ddsd.dwBackBufferCount = 1;
   ddrval = lpDD->CreateSurface( &ddsd, &lpDDSPrimary, NULL );
   if( ddrval != DD_OK ) return DD_InitFail(hwnd);

   // Get the Back Buffer.
   ddscaps.dwCaps = DDSCAPS_BACKBUFFER;
   ddrval = lpDDSPrimary->GetAttachedSurface(&ddscaps, &lpDDSBack);
   if( ddrval != DD_OK ) return DD_InitFail(hwnd);

   // Create and set the palette.
   lpDDPal = DDLoadPalette(lpDD, TILES);
   if (lpDDPal) lpDDSPrimary->SetPalette(lpDDPal);

   // Create offscreen surfaces, by loading bitmaps.
   lpDDSTiles = DDLoadBitmap(lpDD, TILES, 0, 0);     // Tile Textures.
   if( lpDDSTiles == NULL ) return DD_InitFail(hwnd);
   DDSetColorKey(lpDDSTiles, RGB(0,0,0));            // Set the color key (black).
   lpDDSPlayer = DDLoadBitmap(lpDD, PLAYER, 0, 0);   // Player Sprites.
   if( lpDDSPlayer == NULL ) return DD_InitFail(hwnd);
   DDSetColorKey(lpDDSPlayer, RGB(0,0,0));           // Set the color key (black).
   lpDDSMonster = DDLoadBitmap(lpDD, MONSTER, 0, 0); // Monster Sprites.
   if( lpDDSMonster == NULL ) return DD_InitFail(hwnd);
   DDSetColorKey(lpDDSMonster, RGB(0,0,0));          // Set the color key (black).
   lpDDSStatus = DDLoadBitmap(lpDD, STATUS, 0, 0);  // Status Sprites.
   if( lpDDSStatus == NULL ) return DD_InitFail(hwnd);
   DDSetColorKey(lpDDSStatus, RGB(0,0,0));          // Set the color key (black).
   lpDDSPickups = DDLoadBitmap(lpDD, PICKUP, 0, 0); // Pickups Sprites.
   if( lpDDSPickups == NULL ) return DD_InitFail(hwnd);
   DDSetColorKey(lpDDSPickups, RGB(0,0,0));         // Set the color key (black).
   lpDDSFont = DDLoadBitmap(lpDD, FONT, 0, 0);      // Pickups Sprites.
   if( lpDDSFont == NULL ) return DD_InitFail(hwnd);
   DDSetColorKey(lpDDSFont, RGB(0,0,0));            // Set the color key (black).

   // Set up DS and Load my WAV's
   if (config.useds) DS_InitOK = DS_Init();
   /*if (DS_InitOK) DS_LoadWAV("ow.wav",0);
   if (DS_InitOK) DS_LoadWAV("gun.wav",1);
   if (DS_InitOK) DS_LoadWAV("sgun.wav",2);*/
   if (DS_InitOK) DS_LoadWAVRcs("ow.wav",0);
   if (DS_InitOK) DS_LoadWAVRcs("gun.wav",1);
   if (DS_InitOK) DS_LoadWAVRcs("sgun.wav",2);
   if (DS_InitOK) if (config.usewav) DS_LoadWAV("mraks.wav",3);

   NumLevels = GetNumberofLevels();

   if (!Load_Level(0)) {
   	PostMessage(main_window_handle,WM_CLOSE,0,0);
      DebugMessage("Unable to Load the Level");
      }
   player.currentlevel=0;

   return true;
}


/*********************************************************************************
*  DD_Fini:  Release all Graphics, Surfaces.                                     *
*  Returns:  None.                                                               *
*********************************************************************************/
void DD_Fini(void)
{

   if( lpDD != NULL ) {
      if( lpDDSPrimary != NULL ) {
         lpDDSPrimary->Release();
         lpDDSPrimary = NULL;
         }
      if( lpDDSTiles != NULL ) {
         lpDDSTiles->Release();
         lpDDSTiles = NULL;
         }
      if( lpDDSPlayer != NULL ) {
         lpDDSPlayer->Release();
         lpDDSPlayer = NULL;
         }
      if( lpDDSMonster != NULL ) {
         lpDDSMonster->Release();
         lpDDSMonster = NULL;
         }
      if( lpDDSStatus != NULL ) {
         lpDDSStatus->Release();
         lpDDSStatus = NULL;
         }
      if( lpDDSPickups != NULL ) {
         lpDDSPickups->Release();
         lpDDSPickups = NULL;
         }
      if( lpDDSFont != NULL ) {
         lpDDSFont->Release();
         lpDDSFont = NULL;
         }
      if( lpDDPal != NULL ) {
         lpDDPal->Release();
         lpDDPal = NULL;
         }
      lpDD->Release();
      lpDD = NULL;
      }
}

/*********************************************************************************
*  FlipScreen:  Transfers Back Buffer to Screen for Buffering.                   *
*  Returns:     DirectX Sucsess or Predefined Error.                             *
*********************************************************************************/
void FlipScreen(void)
{

HRESULT ddrval;

	while( 1 ) {
      ddrval = lpDDSPrimary->Flip( NULL, 0 );
      if( ddrval == DD_OK ) break;
      if( ddrval == DDERR_SURFACELOST ) {
         ddrval = DD_RestoreAll();
         if( ddrval != DD_OK ) break;
         }
      if( ddrval != DDERR_WASSTILLDRAWING ) break;
      }
}

/*********************************************************************************
*  DD_Restore:  Restores Graphics, Surfaces.                                     *
*  Returns:     DirectX Sucsess or Predifined Error.                             *
*********************************************************************************/
HRESULT DD_RestoreAll(void)
{

HRESULT     ddrval;

   ddrval = lpDDSPrimary->Restore();
   if( ddrval == DD_OK ) {
      ddrval = lpDDSTiles->Restore();
      if( ddrval == DD_OK ) DDReLoadBitmap(lpDDSTiles, TILES);
      ddrval = lpDDSPlayer->Restore();
      if( ddrval == DD_OK ) DDReLoadBitmap(lpDDSPlayer, PLAYER);
      ddrval = lpDDSMonster->Restore();
      if( ddrval == DD_OK ) DDReLoadBitmap(lpDDSMonster, MONSTER);
      ddrval = lpDDSStatus->Restore();
      if( ddrval == DD_OK ) DDReLoadBitmap(lpDDSStatus, STATUS);
      ddrval = lpDDSPickups->Restore();
      if( ddrval == DD_OK ) DDReLoadBitmap(lpDDSPickups, PICKUP);
      ddrval = lpDDSFont->Restore();
      if( ddrval == DD_OK ) DDReLoadBitmap(lpDDSFont, FONT);
      }
   return ddrval;
}

/*********************************************************************************
*  DD_InitFail:  Displays Error Message then Quits Program.                      *
*  Returns:      FALSE.                                                          *
*********************************************************************************/
bool DD_InitFail(HWND hwnd)
{

   FiniObjects();
   MessageBox( hwnd, "DirectDraw Init FAILED", TITLE, MB_OK );
   DestroyWindow( hwnd );
   return false;

}

/*********************************************************************************
*  WinMain:  This is where the fun starts.                                       *
*  Returns:  None.                                                               *
*********************************************************************************/
#pragma argsused
int PASCAL WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{

MSG         msg;

   if( !DO_Init( hInstance, nCmdShow ) ) return false;
   while( 1 ) {
      if( PeekMessage( &msg, NULL, 0, 0, PM_NOREMOVE ) ) {
         if( !GetMessage( &msg, NULL, 0, 0 ) ) return msg.wParam;
         TranslateMessage(&msg);
         DispatchMessage(&msg);
         }
      else Critical(); // Call to the Main Loop!
      }
}

/*********************************************************************************
*  DrawBackGround:  Draws the BackGround Tiles.                                  *
*  Params:          None.                                                        *
*  Returns:         None.                                                        *
*********************************************************************************/
void DrawBackGround(void)
{

int i,o;
int tile;
int tiley_temp, tilex_temp;

   tile = (dungeon1[tiley][tilex]*32);
	DrawTiles(0,0,xoff+tile,yoff+0,32+tile,32);
   tiley_temp = tiley;
   for (o=1;o<15;++o) {
      tile = (dungeon1[tiley_temp+o][tilex]*32);
      DrawTiles(0,(o*32)-yoff,xoff+tile,0,tile+32,32);
      }
   tilex_temp = tilex;
   for (i=1;i<21;++i) {
      tile = (dungeon1[tiley][tilex_temp+i]*32);
      DrawTiles((i*32)-xoff,0,tile,yoff,tile+32,32);
      }
   tilex_temp = tilex;
   tiley_temp = tiley;
   for (i=1;i<21;++i) {
      for (o=1;o<15;++o) {
         tile = (dungeon1[tiley_temp+o][tilex_temp+i]*32);
         DrawTiles((i*32)-xoff,(o*32)-yoff,tile,0,tile+32,32);
         }
      }
}

/*********************************************************************************
*  DrawTiles:  Crummy Tile Drawing Blit function.                                *
*  Params:     posx, posy=onscreen; x1, y1=top left offscreen; x2, y2=bottom     *
*                 right offscreen; transparent=Draw Transparent?.                *
*  Returns:    None.                                                             *
*********************************************************************************/
void DrawTiles(int posx,int posy,int x1,int y1,int x2,int y2)
{

RECT                rcRect; // DirectDraw blitting variable.
HRESULT             ddrval;

   if (posx>608) x2=x2-(posx-608);   //Basic X Clipping
   if (posy>448) y2=y2-(posy-448);   //Basic Y Clipping
   while ( 1 ){
      // Position of the graphic on the offscreen surface.
      rcRect.left = x1;
      rcRect.top = y1;
      rcRect.right = x2;
      rcRect.bottom = y2;
      ddrval = lpDDSBack->BltFast( posx, posy, lpDDSTiles, &rcRect, DDBLTFAST_NOCOLORKEY );
       // If the drawing went OK, then exit the function.
      if( ddrval == DD_OK ) break;
      // Reload the graphics if they got messed up.
      if( ddrval == DDERR_SURFACELOST ) {
         ddrval = DD_RestoreAll();
         if( ddrval != DD_OK ) return;
         }
      // This keeps the drawing in sync with the monitor's refresh rate.
      if( ddrval != DDERR_WASSTILLDRAWING ) return;
      }
}

/*********************************************************************************
*  DrawPlayer:  Crummy Player Drawing Blit function.                             *
*  Params:      None.                                                            *
*  Returns:     None.                                                            *
*********************************************************************************/
void DrawPlayer(void)
{

RECT                rcRect; // DirectDraw blitting variable.
HRESULT             ddrval;
int x2=32, y2=32;

   if (player.x<0) player.x=0;
   if (player.y<0) player.y=0;
   if (player.x>608) player.x=608;
   if (player.y>448) player.y=448;
   while ( 1 ){
      // Position of the graphic on the offscreen surface.
      rcRect.left = player.direction*32;
      rcRect.top = player.animnum*32;
      rcRect.right = (player.direction*32)+x2;
      rcRect.bottom = (player.animnum*32)+y2;
      ddrval = lpDDSBack->BltFast( player.x, player.y, lpDDSPlayer, &rcRect, DDBLTFAST_SRCCOLORKEY );
      if( ddrval == DD_OK ) break;
      if( ddrval == DDERR_SURFACELOST ) {
         ddrval = DD_RestoreAll();
         if( ddrval != DD_OK ) return;
         }
      if( ddrval != DDERR_WASSTILLDRAWING ) return;
      }
}

/*********************************************************************************
*  DrawMonsters:  Crummy Monster Drawing Blit function.                          *
*  Params:        None.                                                          *
*  Returns:       None.                                                          *
*********************************************************************************/
void DrawMonsters(void)
{

RECT                rcRect; // DirectDraw blitting variable.
HRESULT             ddrval;
int i;
int x,y;

   for (i=0;i<100;++i){
      if (monsters[i].active) {
         // Position of the graphic on the offscreen surface.
     	   rcRect.left = monsters[i].direction*32;
      	rcRect.top = monsters[i].animnum*32;
     	   rcRect.right = (monsters[i].direction*32)+32;
      	rcRect.bottom = (monsters[i].animnum*32)+32;
         x = monsters[i].x-xoff-(tilex*32);
         y = monsters[i].y;
         if (x>608) rcRect.right=rcRect.right-(x-608);     //Basic X Clipping
         else if ((x<0) && (x>-32)) { rcRect.left=rcRect.left-x; x=0; }
            while (1) {
            ddrval = lpDDSBack->BltFast( x, y, lpDDSMonster, &rcRect, DDBLTFAST_SRCCOLORKEY );
            if( ddrval == DD_OK ) break;
      	   if( ddrval == DDERR_SURFACELOST ) {
         	   ddrval = DD_RestoreAll();
         	   if( ddrval != DD_OK ) break;
         	   }
      	   if( ddrval != DDERR_WASSTILLDRAWING ) break;
            }
         }
      }
}

/*********************************************************************************
*  DrawPickups:   Crummy Pickups Drawing Blit function.                          *
*  Params:        None.                                                          *
*  Returns:       None.                                                          *
*********************************************************************************/
void DrawPickups(void)
{

RECT                rcRect; // DirectDraw blitting variable.
HRESULT             ddrval;
int i;
int x,y;

   for (i=0;i<100;++i){
      if (pickups[i].active) {
         // Position of the graphic on the offscreen surface.
     	   rcRect.left = pickups[i].type*32;
      	rcRect.top = 0;
     	   rcRect.right = (pickups[i].type*32)+32;
      	rcRect.bottom = 32;
         x = pickups[i].x-xoff-(tilex*32);
         y = pickups[i].y;
         if (x>608) rcRect.right=rcRect.right-(x-608);     //Basic X Clipping
         else if ((x<0) && (x>-32)) { rcRect.left=rcRect.left-x; x=0; }
            while (1) {
            ddrval = lpDDSBack->BltFast( x, y, lpDDSPickups, &rcRect, DDBLTFAST_SRCCOLORKEY );
            if( ddrval == DD_OK ) break;
      	   if( ddrval == DDERR_SURFACELOST ) {
         	   ddrval = DD_RestoreAll();
         	   if( ddrval != DD_OK ) break;
         	   }
      	   if( ddrval != DDERR_WASSTILLDRAWING ) break;
            }
         }
      }
}

/*********************************************************************************
*  DrawBullets:  Crummy Bullet Drawing Blit function.                            *
*  Params:       None.                                                           *
*  Returns:      None.                                                           *
*********************************************************************************/
void DrawBullets(void)
{

RECT                rcRect; // DirectDraw blitting variable.
HRESULT             ddrval;
int i;
int x,y;

   for (i=0;i<100;++i){
      if (bullets[i].active) {
      	// Position of the graphic on the offscreen surface.
      	rcRect.left = 0;
      	rcRect.top = 15;
      	rcRect.right = 2;
      	rcRect.bottom = 17;
         x = bullets[i].x-xoff-(tilex*32);
         y = bullets[i].y;
            while (1) {
            ddrval = lpDDSBack->BltFast( x, y, lpDDSPlayer, &rcRect, DDBLTFAST_NOCOLORKEY );
            if( ddrval == DD_OK ) break;
      	   if( ddrval == DDERR_SURFACELOST ) {
         	   ddrval = DD_RestoreAll();
         	   if( ddrval != DD_OK ) break;
         	   }
      	   if( ddrval != DDERR_WASSTILLDRAWING ) break;
            }
         }
      }
}

/*********************************************************************************
*  DrawStatus:  Crummy Status Drawing Blit function.                             *
*  Params:      None.                                                            *
*  Returns:     None.                                                            *
*********************************************************************************/
void DrawStatus(void)
{

RECT                rcRect; // DirectDraw blitting variable.
HRESULT             ddrval;
char num[20];
int  i;

  	rcRect.left = 0;
   rcRect.top = 0;
   rcRect.right = 16;
   rcRect.bottom = 16;
	for (i=0;i<player.hp;++i) {
   	while (1) {
      	ddrval = lpDDSBack->BltFast( (i*16), 0, lpDDSStatus, &rcRect, DDBLTFAST_SRCCOLORKEY );
         if( ddrval == DD_OK ) break;
      	if( ddrval == DDERR_SURFACELOST ) {
            ddrval = DD_RestoreAll();
            if( ddrval != DD_OK ) break;
            }
      	if( ddrval != DDERR_WASSTILLDRAWING ) break;
         }
      }
   itoa(weapons[player.currentweapon].ammo, num,10);
   if (weapons[player.currentweapon].ammo<10) DrawText(576,0,num,2);
   else DrawText(576,0,num,4);
   for (i=0;i<3;++i) {
   	if (weapons[i].active) {
   		// Position of the graphic on the offscreen surface.
     		rcRect.left = (i*16)+16;
      	rcRect.top = 0;
     		rcRect.right = (i*16)+32;
      	rcRect.bottom = 16;
      	while (1) {
         	ddrval = lpDDSBack->BltFast( 512+(i*16), 0, lpDDSStatus, &rcRect, DDBLTFAST_SRCCOLORKEY );
         	if( ddrval == DD_OK ) break;
      		if( ddrval == DDERR_SURFACELOST ) {
            	ddrval = DD_RestoreAll();
            	if( ddrval != DD_OK ) break;
            	}
      		if( ddrval != DDERR_WASSTILLDRAWING ) break;
         	}
      	}
      }
   // Position of the graphic on the offscreen surface.
   rcRect.left = (player.currentweapon*16)+64;
   rcRect.top = 0;
   rcRect.right = (player.currentweapon*16)+80;
   rcRect.bottom = 16;
   while (1) {
     	ddrval = lpDDSBack->BltFast( 560, 0, lpDDSStatus, &rcRect, DDBLTFAST_SRCCOLORKEY );
     	if( ddrval == DD_OK ) break;
   	if( ddrval == DDERR_SURFACELOST ) {
        	ddrval = DD_RestoreAll();
        	if( ddrval != DD_OK ) break;
        	}
   	if( ddrval != DDERR_WASSTILLDRAWING ) break;
     	}
}

void DrawText(int x, int y, char string[], int size)
{

RECT                rcRect; // DirectDraw blitting variable.
HRESULT             ddrval;
int i, row, col;

	for (i=0;i<size;++i) {
   	switch (string[i]) {
      	case ' ':
            row=0;
            col=0;
            break;
      	case '!':
            row=0;
            col=1;
            break;
      	case '\"':
            row=0;
            col=2;
            break;
      	case '#':
            row=0;
            col=3;
            break;
      	case '$':
            row=0;
            col=4;
            break;
      	case '%':
            row=0;
            col=5;
            break;
      	case '&':
            row=0;
            col=6;
            break;
      	case '\'':
            row=0;
            col=7;
            break;
      	case '(':
            row=0;
            col=8;
            break;
      	case ')':
            row=0;
            col=9;
            break;
      	case '*':
            row=0;
            col=10;
            break;
      	case '+':
            row=0;
            col=11;
            break;
      	case ',':
            row=0;
            col=12;
            break;
      	case '-':
            row=0;
            col=13;
            break;
      	case '.':
            row=0;
            col=14;
            break;
      	case '/':
            row=0;
            col=15;
            break;
      	case '0':
            row=1;
            col=0;
            break;
      	case '1':
            row=1;
            col=1;
            break;
      	case '2':
            row=1;
            col=2;
            break;
      	case '3':
            row=1;
            col=3;
            break;
      	case '4':
            row=1;
            col=4;
            break;
      	case '5':
            row=1;
            col=5;
            break;
      	case '6':
            row=1;
            col=6;
            break;
      	case '7':
            row=1;
            col=7;
            break;
      	case '8':
            row=1;
            col=8;
            break;
      	case '9':
            row=1;
            col=9;
            break;
      	case ':':
            row=1;
            col=10;
            break;
      	case ';':
            row=1;
            col=11;
            break;
      	case '<':
            row=1;
            col=12;
            break;
      	case '=':
            row=1;
            col=13;
            break;
      	case '>':
            row=1;
            col=14;
            break;
      	case '?':
            row=1;
            col=15;
            break;
      	case '@':
            row=2;
            col=0;
            break;
      	case 'A':
            row=2;
            col=1;
            break;
      	case 'B':
            row=2;
            col=2;
            break;
      	case 'C':
            row=2;
            col=3;
            break;
      	case 'D':
            row=2;
            col=4;
            break;
      	case 'E':
            row=2;
            col=5;
            break;
      	case 'F':
            row=2;
            col=6;
            break;
      	case 'G':
            row=2;
            col=7;
            break;
      	case 'H':
            row=2;
            col=8;
            break;
      	case 'I':
            row=2;
            col=9;
            break;
      	case 'J':
            row=2;
            col=10;
            break;
      	case 'K':
            row=2;
            col=11;
            break;
      	case 'L':
            row=2;
            col=12;
            break;
      	case 'M':
            row=2;
            col=13;
            break;
      	case 'N':
            row=2;
            col=14;
            break;
      	case 'O':
            row=2;
            col=15;
            break;
      	case 'P':
            row=3;
            col=0;
            break;
      	case 'Q':
            row=3;
            col=1;
            break;
      	case 'R':
            row=3;
            col=2;
            break;
      	case 'S':
            row=3;
            col=3;
            break;
      	case 'T':
            row=3;
            col=4;
            break;
      	case 'U':
            row=3;
            col=5;
            break;
      	case 'V':
            row=3;
            col=6;
            break;
      	case 'W':
            row=3;
            col=7;
            break;
      	case 'X':
            row=3;
            col=8;
            break;
      	case 'Y':
            row=3;
            col=9;
            break;
      	case 'Z':
            row=3;
            col=10;
            break;
      	case '[':
            row=3;
            col=11;
            break;
      	case '\134':
            row=12;
            col=2;
            break;
      	case ']':
            row=3;
            col=13;
            break;
      	case '^':
            row=3;
            col=14;
            break;
      	case '_':
            row=3;
            col=15;
            break;
      	case '`':
            row=4;
            col=0;
            break;
      	case 'a':
            row=4;
            col=1;
            break;
      	case 'b':
            row=4;
            col=2;
            break;
      	case 'c':
            row=4;
            col=3;
            break;
      	case 'd':
            row=4;
            col=4;
            break;
      	case 'e':
            row=4;
            col=5;
            break;
      	case 'f':
            row=4;
            col=6;
            break;
      	case 'g':
            row=4;
            col=7;
            break;
      	case 'h':
            row=4;
            col=8;
            break;
      	case 'i':
            row=4;
            col=9;
            break;
      	case 'j':
            row=4;
            col=10;
            break;
      	case 'k':
            row=4;
            col=11;
            break;
      	case 'l':
            row=4;
            col=12;
            break;
      	case 'm':
            row=4;
            col=13;
            break;
      	case 'n':
            row=4;
            col=14;
            break;
      	case 'o':
            row=4;
            col=15;
            break;
      	case 'p':
            row=5;
            col=0;
            break;
      	case 'q':
            row=5;
            col=1;
            break;
      	case 'r':
            row=5;
            col=2;
            break;
      	case 's':
            row=5;
            col=3;
            break;
      	case 't':
            row=5;
            col=4;
            break;
      	case 'u':
            row=5;
            col=5;
            break;
      	case 'v':
            row=5;
            col=6;
            break;
      	case 'w':
            row=5;
            col=7;
            break;
      	case 'x':
            row=5;
            col=8;
            break;
      	case 'y':
            row=5;
            col=9;
            break;
      	case 'z':
            row=5;
            col=10;
            break;
      	case '{':
            row=5;
            col=11;
            break;
      	case '|':
            row=5;
            col=12;
            break;
      	case '}':
            row=5;
            col=13;
            break;
      	case '~':
            row=5;
            col=14;
            break;
         case '\n':
         	row=5;
            col=15;
            i=size;
            break;
         default:
            row=5;
            col=15;
            break;
         }
      rcRect.top=row*16;
      rcRect.left=col*16;
      rcRect.right=(col*16)+16;
      rcRect.bottom=(row*16)+16;
   	while (1) {
      	ddrval = lpDDSBack->BltFast( (i*16)+x, y, lpDDSFont, &rcRect, DDBLTFAST_SRCCOLORKEY );
         if( ddrval == DD_OK ) break;
      	if( ddrval == DDERR_SURFACELOST ) {
            ddrval = DD_RestoreAll();
            if( ddrval != DD_OK ) break;
            }
      	if( ddrval != DDERR_WASSTILLDRAWING ) break;
         }
      }
}

/*********************************************************************************
*  PlayerInit:  Player Initializations.                                          *
*  Returns:     None.                                                            *
*********************************************************************************/
void PlayerInit(void)
{

	player.hp=10;
   player.currentweapon=0;
   weapons[Gun].ammo=0;
   weapons[Gun].active=false;
   weapons[Gun].pausecycles=6;
   weapons[Gun].sound=GunShot;
   weapons[Gun].damage=1;
   weapons[Gun].speed=8;
   weapons[ShotGun].ammo=0;
   weapons[ShotGun].active=false;
   weapons[ShotGun].pausecycles=15;
   weapons[ShotGun].sound=SGunShot;
   weapons[ShotGun].damage=10;
   weapons[ShotGun].speed=8;
   weapons[MGun].ammo=0;
   weapons[MGun].active=false;
   weapons[MGun].pausecycles=0;
   weapons[MGun].sound=GunShot;
   weapons[MGun].damage=2;
   weapons[MGun].speed=16;
   if (config.usecd) if (ReadCD()>0) PlayCD(config.cdtrack);
   if (DS_InitOK) if (config.usewav) DS_Play(3);
   if (config.usemidi) {
   	Open_Music();
   	Start_Or_Restart_Track();
      }
}

/*********************************************************************************
*  GameInit:  Game Initializations.                                              *
*  Returns:   None.                                                              *
*********************************************************************************/
void GameInit(void)
{

int i;

	player.x=32;
   player.y=384;
   player.direction=1;
   player.animnum=0;
   player.jumping=false;
   player.active=true;
   player.invonerable=false;
   player.jumpstate=0;
   player.lastshot=0;
   player.invonerablestate=0;
   for (i=0;i<100;i++) monsters[i].active=false;
   for (i=0;i<100;i++) pickups[i].active=false;
   if (!Load_Level(player.currentlevel)) PostMessage(main_window_handle,WM_CLOSE,0,0);
   tilex=0;
   tiley=0;
   xoff=0;
   yoff=0;
   can_scroll_left=false;
   can_scroll_right=true;
   for (i=0;i<100;i++) bullets[i].active=false;
   Timer_Init();
}

/*********************************************************************************
*  Critical:  Main Game Loop.  Runs Title Screen or Game Screen                  *
*  Returns:   None.                                                              *
*********************************************************************************/
void Critical()
{

   while( 1 ) {
      if ((KEY_DOWN(VK_SPACE)) && (GameState == 1)) {
         GameState = 2;
         GameInit();
         PlayerInit();
         }
      if( GameState == 1) {
         TitleUpdateScreen();
         break;
         }
      if( GameState == 2) {
         GameUpdateScreen();
         break;
         }
      }
}

/*********************************************************************************
*  TitleUpdateScreen:  Updates the graphics for the Title or Menu screen.        *
*  Returns:            None.                                                     *
*********************************************************************************/
void TitleUpdateScreen()
{

int i, o;

   for (i=0;i<20;++i) for (o=0;o<15;++o) DrawTiles((i*32),(o*32),dungeon1[o][i]*32,0,(dungeon1[o][i]*32)+32,32);
   DrawText(8,16,"Mindlessly Running Around Killing Stuff",39);
   DrawText(208,64,"By: Tim Wright",14);
   DrawText(160,112,"SpriteLib License #:",20);
   DrawText(144,160,"198.69.77.74-903827629",22);
   DrawText(136,208,"Press SpaceBar to Play!",23);
   FlipScreen();                       // Flip.
}

/*********************************************************************************
*  GameUpdateScreen:  Updates the graphics for the Game.                         *
*  Returns:           None.                                                      *
*********************************************************************************/
void GameUpdateScreen()
{

int i, timer;
char fps[20];

if (KEY_DOWN(VK_ESCAPE)) PostMessage(main_window_handle,WM_CLOSE,0,0);
Timer_Time();
for (timer=0;timer<Timer.Count;++timer) {
	if (!player.active) { GameInit(); PlayerInit(); }
   if (KEY_DOWN('M')) {
   	if (config.usemidi) Start_Or_Restart_Track();
      if (config.usecd) if (ReadCD()>0) PlayCD(config.cdtrack);
      if (DS_InitOK) if (config.usewav) {
      	DS_Stop(3);
         DS_Play(3);
         }
      }
   if (KEY_DOWN('N')) {
   	if (config.usemidi) Stop_Music();
      if (DS_InitOK) if (config.usewav) DS_Stop(3);
      if (config.usecd) StopCD();
      }
   if (KEY_DOWN('R')) player.active=false;
   if ((KEY_DOWN('1')) && (weapons[0].ammo>0) && (weapons[0].active)) player.currentweapon=Gun;
   if ((KEY_DOWN('2')) && (weapons[1].ammo>0) && (weapons[1].active)) player.currentweapon=ShotGun;
   if ((KEY_DOWN('3')) && (weapons[2].ammo>0) && (weapons[2].active)) player.currentweapon=MGun;
   if ((KEY_DOWN(VK_SPACE)) && (player.lastshot==0) && (weapons[player.currentweapon].ammo>0)){
      i=0;
      while(i<100) {
         if (!bullets[i].active) {
         	bullets[i].active=true;
         	if (player.direction==0) bullets[i].x=player.x+(tilex*32)+xoff;
            else bullets[i].x=player.x+(tilex*32)+32+xoff;
            bullets[i].y=player.y+15;
            bullets[i].direction=player.direction;
            bullets[i].type=player.currentweapon;
            player.lastshot=weapons[player.currentweapon].pausecycles;
            if (DS_InitOK) DS_Stop(weapons[player.currentweapon].sound);
      		if (DS_InitOK) DS_Play(weapons[player.currentweapon].sound);
            weapons[player.currentweapon].ammo-=1;
            break;
            }
         else i++;
         }
   	}
   if ((KEY_DOWN(VK_UP)) && (!player.jumping)) {
      if ((dungeon1[((player.y-yoff)/32)+tiley+1][((player.x+xoff)/32)+tilex]>=10) || (dungeon1[((player.y-yoff)/32)+tiley+1][((player.x+xoff)/32)+tilex+1]>=10)) {
         player.jumping=true;
         player.jumpstate=8;
         }
      }
   if (player.jumping) {
      player.jumpstate-=1;
      player.y-=4;
      if (player.jumpstate<0) {
        	player.y+=4;
         player.jumping=false;
         player.jumpstate=0;
         }
      }
   player.lastshot-=1;
   if (player.lastshot<0) player.lastshot=0;
   if (player.invonerable) player.invonerablestate-=1;
   if (player.invonerablestate<0) player.invonerable=false;
   if ((!player.jumping) && (dungeon1[((player.y-yoff)/32)+tiley+1][((player.x+xoff)/32)+tilex]<10) && (dungeon1[((player.y-yoff)/32)+tiley+1][((player.x+xoff)/32)+tilex+1]<10)) player.y+=4;
   if ((KEY_DOWN(VK_LEFT)) && (dungeon1[((player.y-yoff+28)/32)+tiley][((player.x+xoff-4)/32)+tilex]<10) && (dungeon1[((player.y-yoff)/32)+tiley][((player.x+xoff-4)/32)+tilex]<10)) {
   	player.direction = 0;
      player.animnum+=1;
      if (player.animnum>1) player.animnum = 0;
      if ((!can_scroll_left) || (player.x>312)) player.x-=4;
      else {
         xoff-=4;
         can_scroll_right=true;
         if (xoff<4) {
            xoff=32;
            --tilex;
            if (tilex<0) {
               xoff=0;
               tilex=0;
               can_scroll_left=false;
               }
            }
         }
      }
   if ((KEY_DOWN(VK_RIGHT)) && (dungeon1[((player.y-yoff+28)/32)+tiley][((player.x+xoff+4)/32)+tilex+1]<10) && (dungeon1[((player.y-yoff)/32)+tiley][((player.x+xoff+4)/32)+tilex+1]<10)) {
   	player.direction = 1;
      player.animnum+=1;
      if (player.animnum>1) player.animnum = 0;
      if ((!can_scroll_right) || (player.x<328)) player.x+=4;
      else {
         xoff+=4;
         can_scroll_left=true;
         if (xoff>32) {
            xoff=4;
            ++tilex;
            if (tilex>LevelLength) {
               xoff=32;
               tilex=LevelLength;
               can_scroll_right=false;
               }
            }
         }
      }
   MoveMonsters();
   MoveBullets();
   DetectPickups();
   DetectMisc();
   }
   DrawBackGround();
   DrawPlayer();
   DrawMonsters();
   DrawBullets();
   DrawStatus();
   DrawText(0,460,"FPS:",4);
   itoa(Timer.FPS, fps, 10);
   DrawText(64,460,fps,4);
   DetectMisc();
   DrawPickups();
   FlipScreen();
}

void DetectMisc(void)
{

	if (weapons[player.currentweapon].ammo<=0) {
   	if (weapons[MGun].ammo>0) player.currentweapon=MGun;
      else if (weapons[ShotGun].ammo>0) player.currentweapon=ShotGun;
      else if (weapons[Gun].ammo>0) player.currentweapon=Gun;
      }
	if (player.x>600) {
   	player.currentlevel+=1;
      if (player.currentlevel>NumLevels) {
      	DrawText(40,208,"Congrats, You beat this DORKY Demo!",35);
         DrawText(48,240,"Press R to Restart or ESC to Quit.",34);
         player.currentlevel=2;
        	if (KEY_DOWN(VK_ESCAPE)) PostMessage(main_window_handle,WM_CLOSE,0,0);
        	if (KEY_DOWN('R')) {
        		player.currentlevel=0;
            PlayerInit();
           	GameInit();
           	}
        	}
      else GameInit();
      }
}

void DetectPickups(void)
{

int i;

	for (i=0;i<100;++i) {
   	if (pickups[i].active) {
			if (((pickups[i].x+32)>=(player.x+(tilex*32)+xoff)) && (pickups[i].x<=(player.x+(tilex*32)+xoff+32)) && (pickups[i].y>=player.y) && (pickups[i].y<=(player.y+32))) {
   			pickups[i].active=false;
         	if (pickups[i].type==0) player.hp+=1;
            if (pickups[i].type==1) { weapons[Gun].active=true; weapons[Gun].ammo+=15; }
            if (pickups[i].type==2) { weapons[ShotGun].active=true; weapons[ShotGun].ammo+=4; }
            if (pickups[i].type==3) { weapons[MGun].active=true; weapons[MGun].ammo+=30; }
            if (pickups[i].type==4) weapons[Gun].ammo+=15;
            if (pickups[i].type==5) weapons[ShotGun].ammo+=4;
            if (pickups[i].type==6) weapons[MGun].ammo+=30;
            }
         }
      }
}

void MoveMonsters(void)
{

int i;

   for(i=0;i<100;++i) {
   	if (monsters[i].active) {
      	if (((player.x+(tilex*32)+xoff)>monsters[i].x) && (monsters[i].direction==1) && ((monsters[i].x-xoff-(tilex*32))>(-32)) && ((monsters[i].x-xoff-(tilex*32))<640)) monsters[i].engaged=true;
      	if (((player.x+(tilex*32)+xoff)<monsters[i].x) && (monsters[i].direction==0) && ((monsters[i].x-xoff-(tilex*32))>(-32)) && ((monsters[i].x-xoff-(tilex*32))<640)) monsters[i].engaged=true;
      	if (monsters[i].engaged) {
         	if ((player.y<monsters[i].y) && (!monsters[i].jumping)){
            	if ((dungeon1[((monsters[i].y)/32)+1][((monsters[i].x)/32)]>=10) || (dungeon1[((monsters[i].y)/32)+1][((monsters[i].x)/32)+1]>=10)) {
            		monsters[i].jumping=true;
           			monsters[i].jumpstate=8;
               	}
           	   }
	         	if (monsters[i].jumping) {
   	         monsters[i].jumpstate-=1;
      			monsters[i].y-=4;
      			if (monsters[i].jumpstate<0) {
         			monsters[i].y+=4;
         			monsters[i].jumping=false;
	         		monsters[i].jumpstate=0;
   	      		}
     				}
         	if (((player.x+(tilex*32)+xoff)>monsters[i].x) && (dungeon1[((monsters[i].y+28)/32)][((monsters[i].x+4)/32)+1]<10) && (dungeon1[((monsters[i].y)/32)][((monsters[i].x+4)/32)+1]<10)) {
            	monsters[i].x+=2;
            	monsters[i].direction=1;
            	monsters[i].animnum+=1;
         		if (monsters[i].animnum>1) monsters[i].animnum=0;
            	}
         	else if (((player.x+(tilex*32)+xoff)<monsters[i].x) && (dungeon1[((monsters[i].y+28)/32)][((monsters[i].x-4)/32)]<10) && (dungeon1[((monsters[i].y)/32)][((monsters[i].x-4)/32)]<10)) {
            	monsters[i].x-=2;
            	monsters[i].direction=0;
            	monsters[i].animnum+=1;
            	if (monsters[i].animnum>1) monsters[i].animnum=0;
            	}
            }
   		if ((!monsters[i].jumping) && (dungeon1[((monsters[i].y)/32)+1][((monsters[i].x)/32)]<10) && (dungeon1[((monsters[i].y)/32)+1][((monsters[i].x)/32)+1]<10)) monsters[i].y+=4;
         if (((monsters[i].x+32)>=(player.x+(tilex*32)+xoff)) && (monsters[i].x<=(player.x+(tilex*32)+xoff+32)) && (monsters[i].y>=player.y) && (monsters[i].y<=(player.y+32)) && (!player.invonerable)) {
            monsters[i].engaged=true;
         	player.hp-=1;
            player.invonerable=true;
            player.invonerablestate=20;
            if (DS_InitOK) DS_Stop(PlayerHurt);
      		if (DS_InitOK) DS_Play(PlayerHurt);
            if (player.hp<=0) player.active=false;
            }
         }
      }
}

void MoveBullets(void)
{

int i,o;

	for (i=0;i<100;++i) {
      if (bullets[i].active) {
         if (bullets[i].direction==1) bullets[i].x+=weapons[bullets[i].type].speed;
         if (bullets[i].x>((LevelLength+21)*32)) bullets[i].active=false;
         if (bullets[i].direction==0) bullets[i].x-=weapons[bullets[i].type].speed;
         if (bullets[i].x<0) bullets[i].active=false;
         if (dungeon1[bullets[i].y/32][bullets[i].x/32]>=10) bullets[i].active=false;
         for (o=0;o<100;++o) {
         	if (monsters[o].active) {
            	if ((bullets[i].x>monsters[o].x) && (bullets[i].x<(monsters[o].x+32)) && (bullets[i].y>monsters[o].y) && (bullets[i].y<(monsters[o].y+32))) {
               	monsters[o].engaged=true;
               	bullets[i].active=false;
                  monsters[o].hp-=weapons[bullets[i].type].damage;
                  if (monsters[o].hp<=0) monsters[o].active=false;
                  }
               }
            }
         }
      }
}

//////****************SHORT MUSIC ROUTINES, THAT USE MCI******************////////
void Open_Music(void)
{

char strCommandString[80];
MCIERROR ret;

	sprintf(strCommandString,"open MRAKS.MID type sequencer alias jazz");
   ret = mciSendString(strCommandString, 0, 0, 0);
	if (ret == 0) music_enabled = TRUE;
}

void Start_Or_Restart_Track(void)
{

MCIERROR ret;

	if (music_enabled)  ret = mciSendString("play jazz from 0", 0, 0, 0);
   if (ret) music_enabled=FALSE;
}

void Stop_Music(void)
{

	if (music_enabled) mciSendString("stop jazz", 0, 0, 0);
}

void Close_Music(void)
{

	if (music_enabled) {
		mciSendString("stop jazz", 0, 0, 0);
		mciSendString("close jazz", 0, 0, 0);
		}
}
